--- old/usr/src/cmd/hal/hald/solaris/devinfo_storage.c	2014-05-05 01:02:55.834033550 +0100
+++ new/usr/src/cmd/hal/hald/solaris/devinfo_storage.c	2014-05-05 01:02:55.844540616 +0100
@@ -4,6 +4,7 @@
  *
  * Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2013 Garrett D'Amore <garrett@damore.org>
+ * Copyright 2014 Andrew Stormont.
  *
  * Licensed under the Academic Free License version 2.1
  *
@@ -39,8 +40,14 @@
 
 #ifdef sparc
 #define	WHOLE_DISK	"s2"
+#define	DOS_SEPERATOR	":"
+#define	DOS_FORMAT	"s2:%d"
+#define	DOS_TEMPLATE	":NN"
 #else
 #define	WHOLE_DISK	"p0"
+#define	DOS_SEPERATOR	"p"
+#define	DOS_FORMAT	"p%d"
+#define	DOS_TEMPLATE	"pNN"
 #endif
 
 /* some devices,especially CDROMs, may take a while to be probed (values in ms) */
@@ -702,11 +709,11 @@
 	devinfo_volume_add (d, node, m);
 	devinfo_storage_free_minor (m);
 
-	doslink = (char *)calloc (1, strlen (devlink) + sizeof (":NNN") + 1);
+	doslink = (char *)calloc (1, strlen (devlink) + sizeof (DOS_TEMPLATE) + 1);
 	if (doslink != NULL) {
 		for (i = 1; i < 16; i++) {
-			snprintf(dospath, sizeof (dospath), WHOLE_DISK":%d", i);
-			sprintf(doslink, "%s:%d", devlink, i);
+			snprintf(dospath, sizeof (dospath), DOS_FORMAT, i);
+			sprintf(doslink, "%s"DOS_SEPERATOR"%d", devlink, i);
 			m = devinfo_storage_new_minor(minor_path, dospath, doslink, dev, i);
 			devinfo_volume_add (d, node, m);
 			devinfo_storage_free_minor (m);
@@ -859,14 +866,6 @@
 			continue;
 		}
 
-		/* ignore p1..N - we'll use p0:N instead */
-		if ((strlen (slice) > 1) && (slice[0] == 'p') && isdigit(slice[1]) &&
-		    ((atol(&slice[1])) > 0)) {
-			free (devlink);
-			di_devfs_path_free (minor_path);
-			continue;
-		}
-
 		m = devinfo_storage_new_minor(minor_path, slice, devlink, dev, -1);
 		if (m == NULL) {
 			free (devlink);
@@ -915,11 +914,11 @@
 
 	/* add virtual dos volumes to enable pcfs probing */
 	if (!is_cdrom) {
-		doslink_len = strlen (maindev->devlink) + sizeof (":NNN") + 1;
+		doslink_len = strlen (maindev->devlink) + sizeof (DOS_TEMPLATE) + 1;
 		if ((doslink = (char *)calloc (1, doslink_len)) != NULL) {
 			for (i = 1; i < 16; i++) {
-				snprintf(dospath, sizeof (dospath), "%s:%d", maindev->slice, i);
-				snprintf(doslink, doslink_len, "%s:%d", maindev->devlink, i);
+				snprintf(dospath, sizeof (dospath), "%s"DOS_SEPERATOR"%d", maindev->slice, i);
+				snprintf(doslink, doslink_len, "%s"DOS_SEPERATOR"%d", maindev->devlink, i);
 				m = devinfo_storage_new_minor(maindev_path, dospath, doslink, maindev->dev, i);
 				g_queue_push_tail (mq, m);
 			}
@@ -1403,7 +1402,7 @@
 {
 	char *p;
 
-	if ((p = strrchr (path, ':')) == NULL) {
+	if ((p = strrchr (path, DOS_SEPERATOR)) == NULL) {
 		return (FALSE);
 	}
 	return ((*partnum = atoi(p + 1)) != 0);
@@ -1414,7 +1413,7 @@
 {
 	char *p;
 
-	if ((p = strrchr (path, ':')) == NULL) {
+	if ((p = strrchr (path, DOS_SEPERATOR)) == NULL) {
 		return (FALSE);
 	}
 	if ((*partnum = atoi(p + 1)) == 0) {
@@ -1422,7 +1421,7 @@
 	}
 	p[0] = '\0';
 	*devpath = strdup(path);
-	p[0] = ':';
+	p[0] = DOS_SEPERATOR;
 	return (*devpath != NULL);
 }
 
